<?php

namespace App\EventSubscriber;

use App\Entity\Capsule;
use App\Entity\PendingEditorInvitation;
use App\Entity\User;
use App\Event\UserRegistrationEvent;
use App\Repository\CapsuleRepository;
use App\Repository\PendingEditorInvitationRepository;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class UserRegistrationSubscriber implements EventSubscriberInterface
{
    public function __construct(
        private PendingEditorInvitationRepository $pending_editor_invitation_repository,
        private CapsuleRepository $capsule_repository,
        private EntityManagerInterface $entity_manager
    ) {
    }

    public static function getSubscribedEvents(): array
    {
        return [
            UserRegistrationEvent::class => [
                'onPendingEditorRegistration', 1
            ]
        ];
    }

    public function onPendingEditorRegistration(UserRegistrationEvent $event): void
    {
        $user = $event->getUser();

        $this->addCapsuleToConfirmedPreviousPendingEditor($user);
    }

    public function addCapsuleToConfirmedPreviousPendingEditor(User $user): void
    {
        $pending_editor_invitations = $this->pending_editor_invitation_repository->findBy(
            ['email' => $user->getEmail()]
        );

        if (! is_array($pending_editor_invitations)) {
            return;
        }

        $capsules_ids = $this->getCapsulesIds($pending_editor_invitations);
        $this->removeInvitations($pending_editor_invitations);
        $this->addEditorToCapsules($capsules_ids, $user);
    }

    /**
     * @param PendingEditorInvitation[] $pending_editor_invitations
     * @return int[]
     */
    private function getCapsulesIds(array $pending_editor_invitations): array
    {
        $capsules_ids = [];

        foreach ($pending_editor_invitations as $invitation) {
            $capsules_ids[] = $invitation->getCapsuleId();
        }

        return $capsules_ids;
    }

    /**
     * @param PendingEditorInvitation[] $pending_editor_invitations
     */
    private function removeInvitations(array $pending_editor_invitations): void
    {
        foreach ($pending_editor_invitations as $invitation) {
            $this->entity_manager->remove($invitation);
        }
    }

    /**
     * @param array<int> $capsules_ids
     */
    private function addEditorToCapsules(array $capsules_ids, User $user): void
    {
        $capsules = $this->capsule_repository->findBy(['id' => $capsules_ids]);

        foreach ($capsules as $capsule) {
            if (! $capsule instanceof Capsule) {
                return;
            }
            $capsule->addEditor($user);
            $this->entity_manager->persist($capsule);
        }
        $this->entity_manager->flush();
    }
}
