<?php

namespace App\Repository;

use App\Entity\User;
use Doctrine\Bundle\DoctrineBundle\Repository\ServiceEntityRepository;
use Doctrine\Persistence\ManagerRegistry;
use Symfony\Component\Security\Core\Exception\UnsupportedUserException;
use Symfony\Component\Security\Core\User\PasswordAuthenticatedUserInterface;
use Symfony\Component\Security\Core\User\PasswordUpgraderInterface;

/**
 * @method User|null find($id, $lockMode = null, $lockVersion = null)
 * @method User|null findOneBy(array $criteria, array $orderBy = null)
 * @method User[]    findAll()
 * @method User[]    findBy(array $criteria, array $orderBy = null, $limit = null, $offset = null)
 * @template UserEntity of object
 * @extends ServiceEntityRepository<UserEntity>
 */
class UserRepository extends ServiceEntityRepository implements PasswordUpgraderInterface
{
    public function __construct(ManagerRegistry $registry)
    {
        parent::__construct($registry, User::class);
    }

    /**
     * Used to upgrade (rehash) the user's password automatically over time.
     */
    public function upgradePassword(PasswordAuthenticatedUserInterface $user, string $newHashedPassword): void
    {
        if (!$user instanceof User) {
            throw new UnsupportedUserException(sprintf('Instances of "%s" are not supported.', \get_class($user)));
        }

        $user->setPassword($newHashedPassword);
        $this->_em->persist($user);
        $this->_em->flush();
    }

    public function findOneByEmail(string $value): ?User
    {
        return $this->createQueryBuilder('u')
            ->andWhere('u.email = :val')
            ->setParameter('val', $value)
            ->getQuery()
            ->getOneOrNullResult()
        ;
    }

    public function getUserNameFromUserId(int $user_id): int
    {
        return $this->createQueryBuilder('u.firstname')
            ->andWhere('u.id = :id')
            ->setParameter('id', $user_id)
            ->getQuery()
            ->getOneOrNullResult()
            ;
    }

    /**
     * @param array<int> $user_ids
     * @return array<User>
     */
    public function getUsersFromIds(array $user_ids): array
    {
        return $this->createQueryBuilder('u')
            ->andWhere('u.id in (:ids)')
            ->setParameter('ids', $user_ids)
            ->getQuery()
            ->getResult();
    }

    public function emailExists(string $email): bool
    {
        return $this->createQueryBuilder('r')
                ->andWhere('r.email = :email')
                ->setParameter('email', $email)
                ->getQuery()
                ->getOneOrNullResult() !== null;
    }
}
